@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # vim:ts=4:sw=4:softtabstop=4:smarttab:expandtab
 
-# Copyright (c) 2014-2019 Kevin B. Hendricks and Doug Massay
+# Copyright (c) 2014-2020 Kevin B. Hendricks and Doug Massay
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -29,7 +29,7 @@ from __future__ import unicode_literals, division, absolute_import, print_functi
 
 
 from compatibility_utils import PY3, PY2, text_type, binary_type, utf8_str, unicode_str, iswindows
-
+from collections import OrderedDict
 import sys
 import os
 
@@ -41,7 +41,7 @@ import unipath
 from unipath import pathof
 import unicodedata
 
-_launcher_version=20191216
+_launcher_version=20200326
 
 _PKG_VER = re.compile(r'''<\s*package[^>]*version\s*=\s*["']([^'"]*)['"][^>]*>''',re.IGNORECASE)
 
@@ -100,7 +100,9 @@ class Wrapper(object):
         # status of epub inside Sigil (isDirty) and CurrentFilePath of current epub file
         self.epub_isDirty = False
         self.epub_filepath = ""
-        # File selected in Sigil's Book Browser
+        self.colormode = None
+        self.colors = None
+         # File selected in Sigil's Book Browser
         self.selected = []
         cfg = ''
         with open(os.path.join(self.outdir, 'sigil.cfg'), 'rb') as f:
@@ -117,18 +119,22 @@ class Wrapper(object):
             self.sigil_spellcheck_lang = cfg_lst.pop(0)
             self.epub_isDirty = (cfg_lst.pop(0) == "True")
             self.epub_filepath = cfg_lst.pop(0)
+            self.colormode = cfg_lst.pop(0)
+            self.colors = cfg_lst.pop(0)
+            self.highdpi = cfg_lst.pop(0)
+            self.uifont = cfg_lst.pop(0)
             self.selected = cfg_lst
         os.environ['SigilGumboLibPath'] = self.get_gumbo_path()
 
         # dictionaries used to map opf manifest information
-        self.id_to_href = {}
-        self.id_to_mime = {}
-        self.id_to_props = {}
-        self.id_to_fall = {}
-        self.id_to_over = {}
-        self.id_to_bookpath = {}
-        self.href_to_id = {}
-        self.bookpath_to_id = {}
+        self.id_to_href = OrderedDict()
+        self.id_to_mime = OrderedDict()
+        self.id_to_props = OrderedDict()
+        self.id_to_fall = OrderedDict()
+        self.id_to_over = OrderedDict()
+        self.id_to_bookpath = OrderedDict()
+        self.href_to_id = OrderedDict()
+        self.bookpath_to_id = OrderedDict()
         self.spine_ppd = None
         self.spine = []
         self.guide = []
@@ -157,14 +163,18 @@ class Wrapper(object):
             self.bindings = op.get_bindings()
             self.metadataxml = op.get_metadataxml()
             # invert key dictionaries to allow for reverse access
-            self.href_to_id = {v: k for k, v in self.id_to_href.items()}
-            self.bookpath_to_id = {v: k for k, v in self.id_to_bookpath.items()}
+            for k, v in self.id_to_href.items():
+                self.href_to_id[v] = k
+            for k, v in self.id_to_bookpath.items():
+                self.bookpath_to_id[v] = k
+            # self.href_to_id = {v: k for k, v in self.id_to_href.items()}
+            # self.bookpath_to_id = {v: k for k, v in self.id_to_bookpath.items()}
             # self.metadata = op.get_metadata()
             # self.metadata_attr = op.get_metadata_attr()
         self.other = []  # non-manifest file information
-        self.id_to_filepath = {}
-        self.book_href_to_filepath = {}
-        self.modified = {}
+        self.id_to_filepath = OrderedDict()
+        self.book_href_to_filepath = OrderedDict()
+        self.modified = OrderedDict()
         self.added = []
         self.deleted = []
 
@@ -200,6 +210,24 @@ class Wrapper(object):
         return ext_mime_map.get(ext, "")
 
 
+    # New in Sigil 1.1
+    # ------------------
+
+    # returns color mode of Sigil "light" or "dark"
+    def colorMode(self):
+        return unicode_str(self.colormode)
+
+    # returns color as css or javascript hex color string #xxxxxx
+    # Accepts the following color roles "Window", "Base", "Text", "Highlight", "HighlightedText"
+    def color(self, role):
+        role = unicode_str(role)
+        role = role.lower()
+        color_roles = ["window", "base", "text", "highlight", "highlightedtext"]
+        colors = self.colors.split(',')
+        if role in color_roles:
+            idx = color_roles.index(role)
+            return unicode_str(colors[idx])
+        return None
 
     # New in Sigil 1.0
     # ----------------
